@@ -182,7 +182,7 @@ MediaMeta MetaDetector::updateMetaFromLocalfile(MediaMeta meta, const QFileInfo
     auto current = QDateTime::currentDateTime();
     // HACK how to sort by add time
     meta.timestamp = current.toMSecsSinceEpoch() * 1000;
-    meta.filetype = fileInfo.suffix();
+    meta.filetype = fileInfo.suffix().toLower(); //歌曲文件后缀使用小写保存
 
     if (meta.title.isEmpty()) {
         meta.title = fileInfo.completeBaseName();
@@ -351,11 +351,7 @@ void MetaDetector::getCoverData(const QString &path, const QString &tmpPath, con
             QBuffer buffer(&byteArray);
             buffer.open(QIODevice::WriteOnly);
             image.save(&buffer, "jpg");
-#ifdef TABLET_PC
-            image = image.scaled(QSize(200, 200));
-#else
-            image = image.scaled(QSize(160, 160));
-#endif
+            image = image.scaled(QSize(200, 200), Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation);
             image.save(imagesDirPath + "/" + imageName);
         } else {
 //            image = QImage(":/common/image/cover_max.svg");
