@@ -49,22 +49,22 @@ DWIDGET_USE_NAMESPACE
 // 升序
 bool moreThanTimestampASC(AlbumInfo v1, AlbumInfo v2)
 {
-    return v1.timestamp <= v2.timestamp;
+    return v1.timestamp < v2.timestamp;
 }
 
 bool moreThanTitleASC(const AlbumInfo v1, const AlbumInfo v2)
 {
-    return v1.pinyinAlbum <= v2.pinyinAlbum;
+    return v1.pinyinAlbum < v2.pinyinAlbum;
 }
 // 降序
 bool moreThanTimestampDES(AlbumInfo v1, AlbumInfo v2)
 {
-    return v1.timestamp >= v2.timestamp;
+    return v1.timestamp > v2.timestamp;
 }
 
 bool moreThanTitleDES(const AlbumInfo v1, const AlbumInfo v2)
 {
-    return v1.pinyinAlbum >= v2.pinyinAlbum;
+    return v1.pinyinAlbum > v2.pinyinAlbum;
 }
 
 int calculateAlbumSize(int index, AlbumInfo info)
@@ -159,14 +159,14 @@ AlbumListView::AlbumListView(const QString &hash, QWidget *parent)
             this, &AlbumListView::slotUpdateCodec);
     // 删除歌曲
     connect(DataBaseService::getInstance(), &DataBaseService::sigRemoveSelectedSongs, this, &AlbumListView::slotRemoveSelectedSongs, Qt::DirectConnection);
-#ifdef TABLET_PC
-    connect(Player::getInstance(), &Player::signalPlaybackStatusChanged,
-            this, &AlbumListView::slotPlaybackStatusChanged);
+    if (CommonService::getInstance()->isTabletEnvironment()) {
+        connect(Player::getInstance(), &Player::signalPlaybackStatusChanged,
+                this, &AlbumListView::slotPlaybackStatusChanged);
 
-    // 横竖屏切换
-    connect(CommonService::getInstance(), &CommonService::signalHScreen,
-            this, &AlbumListView::slotHScreen);
-#endif
+        // 横竖屏切换
+        connect(CommonService::getInstance(), &CommonService::signalHScreen,
+                this, &AlbumListView::slotHScreen);
+    }
 }
 
 AlbumListView::~AlbumListView()
@@ -356,42 +356,42 @@ int AlbumListView::getAlbumCount()
 
 void AlbumListView::setViewModeFlag(QListView::ViewMode mode)
 {
-#ifdef TABLET_PC
-    if (mode == QListView::IconMode) {
-        setIconSize(QSize(150, 150));
-        setGridSize(QSize(-1, -1));
-        // 去除底部间距
-        setViewportMargins(30, -13, -35, 0);
-        setSpacing(20);
-        if (CommonService::getInstance()->isHScreen()) {
-            setSpacing(20);
+    if (CommonService::getInstance()->isTabletEnvironment()) {
+        if (mode == QListView::IconMode) {
+            setIconSize(QSize(150, 150));
+            setGridSize(QSize(-1, -1));
+            // 去除底部间距
             setViewportMargins(30, -13, -35, 0);
+            setSpacing(20);
+            if (CommonService::getInstance()->isHScreen()) {
+                setSpacing(20);
+                setViewportMargins(30, -13, -35, 0);
+            } else {
+                setSpacing(33);
+                setViewportMargins(23, -13, -35, 0);
+            }
         } else {
-            setSpacing(33);
-            setViewportMargins(23, -13, -35, 0);
+            setIconSize(QSize(36, 36));
+            setGridSize(QSize(-1, -1));
+            // 修改间距
+            setViewportMargins(10, 0, 10, 0);
+            setSpacing(0);
         }
     } else {
-        setIconSize(QSize(36, 36));
-        setGridSize(QSize(-1, -1));
-        // 修改间距
-        setViewportMargins(10, 0, 10, 0);
-        setSpacing(0);
+        if (mode == QListView::IconMode) {
+            setIconSize(QSize(150, 150));
+            setGridSize(QSize(-1, -1));
+            // 去除底部间距
+            setViewportMargins(-10, -13, -35, 0);
+            setSpacing(20);
+        } else {
+            setIconSize(QSize(36, 36));
+            setGridSize(QSize(-1, -1));
+            // 修改顶部间距
+            setViewportMargins(0, 0, 8, 0);
+            setSpacing(0);
+        }
     }
-#else
-    if (mode == QListView::IconMode) {
-        setIconSize(QSize(150, 150));
-        setGridSize(QSize(-1, -1));
-        // 去除底部间距
-        setViewportMargins(-10, -13, -35, 0);
-        setSpacing(20);
-    } else {
-        setIconSize(QSize(36, 36));
-        setGridSize(QSize(-1, -1));
-        // 修改顶部间距
-        setViewportMargins(0, 0, 8, 0);
-        setSpacing(0);
-    }
-#endif
 
     setViewMode(mode);
     m_viewModel = mode;
@@ -553,9 +553,17 @@ void AlbumListView::slotRemoveSelectedSongs(const QString &deleteHash, const QSt
     if (deleteHash != "album" || Player::getInstance()->getCurrentPlayListHash() != "album") {
         return;
     }
-    if (musicHashs.size() == 0) {
+    if (musicHashs.size() == 0 || Player::getInstance()->getPlayList()->isEmpty()) {
         return;
     }
+    QList<MediaMeta> *curPlayList = Player::getInstance()->getPlayList();
+    // 歌单存在非歌手歌曲不处理
+    QString firstAlbum = curPlayList->first().album;
+    for (int i = 1; i < curPlayList->size(); i++) {
+        if (curPlayList->at(i).album != firstAlbum) {
+            return;
+        }
+    }
     // 标志准备删除的歌曲,找到当前正在播放的专辑index
     int playIndex = -1;
     for (int i = 0; i < albumModel->rowCount(); i++) {
@@ -593,7 +601,10 @@ void AlbumListView::slotRemoveSelectedSongs(const QString &deleteHash, const QSt
     }
     // 记录切换歌单之前播放状态
     Player::PlaybackStatus preStatue = Player::getInstance()->status();
-    // 找到要播放的专辑
+    // 找到要播放的专辑，如果删除的是最后一个专辑的唯一一首歌，则从第一个播起
+    if (playIndex == (albumModel->rowCount() - 1)) {
+        playIndex = 0;
+    }
     for (int i = playIndex; i < albumModel->rowCount(); i++) {
         QModelIndex idx = albumModel->index(i, 0, QModelIndex());
         AlbumInfo albumTmp = idx.data(Qt::UserRole).value<AlbumInfo>();
@@ -664,7 +675,7 @@ void AlbumListView::dropEvent(QDropEvent *event)
         auto urls = event->mimeData()->urls();
         QStringList localpaths;
         for (auto &url : urls) {
-            localpaths << url.toLocalFile();
+            localpaths << (url.isLocalFile() ? url.toLocalFile() : url.path());
         }
 
         if (!localpaths.isEmpty()) {
@@ -715,9 +726,9 @@ void AlbumListView::setSortType(DataBaseService::ListSortType sortType)
 void AlbumListView::setDataBySortType(QList<AlbumInfo> &albumInfos, DataBaseService::ListSortType sortType)
 {
     // 不加入到搜索结果中
-    if (m_hash == "albumResult" || m_hash == "artistResult") {
-        return;
-    }
+//    if (m_hash == "albumResult" || m_hash == "artistResult") {
+//        return;
+//    }
     // 排序
     sortList(albumInfos, sortType);
 
@@ -816,7 +827,7 @@ bool AlbumListView::mapContainsList(QMap<QString, MediaMeta> metasMap, QStringLi
     }
     return contain;
 }
-#ifdef TABLET_PC
+
 void AlbumListView::slotPlaybackStatusChanged(Player::PlaybackStatus statue)
 {
     Q_UNUSED(statue)
@@ -837,4 +848,3 @@ void AlbumListView::slotHScreen(bool isHScreen)
         }
     }
 }
-#endif
