@@ -47,22 +47,22 @@ DWIDGET_USE_NAMESPACE
 // 升序
 bool moreThanTimestampASC(SingerInfo v1, SingerInfo v2)
 {
-    return v1.timestamp <= v2.timestamp;
+    return v1.timestamp < v2.timestamp;
 }
 
 bool moreThanSingerASC(SingerInfo v1, SingerInfo v2)
 {
-    return v1.pinyinSinger <= v2.pinyinSinger;
+    return v1.pinyinSinger < v2.pinyinSinger;
 }
 // 降序
 bool moreThanTimestampDES(SingerInfo v1, SingerInfo v2)
 {
-    return v1.timestamp >= v2.timestamp;
+    return v1.timestamp > v2.timestamp;
 }
 
 bool moreThanSingerDES(SingerInfo v1, SingerInfo v2)
 {
-    return v1.pinyinSinger >= v2.pinyinSinger;
+    return v1.pinyinSinger > v2.pinyinSinger;
 }
 
 int calculateSingerSize(int index, SingerInfo info)
@@ -162,14 +162,14 @@ SingerListView::SingerListView(const QString &hash, QWidget *parent)
             this, &SingerListView::slotUpdateCodec);
     // 删除歌曲
     connect(DataBaseService::getInstance(), &DataBaseService::sigRemoveSelectedSongs, this, &SingerListView::slotRemoveSelectedSongs, Qt::DirectConnection);
-#ifdef TABLET_PC
-    connect(Player::getInstance(), &Player::signalPlaybackStatusChanged,
-            this, &SingerListView::slotPlaybackStatusChanged);
+    if (CommonService::getInstance()->isTabletEnvironment()) {
+        connect(Player::getInstance(), &Player::signalPlaybackStatusChanged,
+                this, &SingerListView::slotPlaybackStatusChanged);
 
-    // 横竖屏切换
-    connect(CommonService::getInstance(), &CommonService::signalHScreen,
-            this, &SingerListView::slotHScreen);
-#endif
+        // 横竖屏切换
+        connect(CommonService::getInstance(), &CommonService::signalHScreen,
+                this, &SingerListView::slotHScreen);
+    }
 }
 
 SingerListView::~SingerListView()
@@ -352,42 +352,42 @@ int SingerListView::getMusicCount()
 
 void SingerListView::setViewModeFlag(QListView::ViewMode mode)
 {
-#ifdef TABLET_PC
-    if (mode == QListView::IconMode) {
-        setIconSize(QSize(200, 200));
-        setGridSize(QSize(-1, -1));
-        // 去除底部间距
-        setViewportMargins(30, -13, -35, 0);
-        setSpacing(20);
-        if (CommonService::getInstance()->isHScreen()) {
-            setSpacing(20);
+    if (CommonService::getInstance()->isTabletEnvironment()) {
+        if (mode == QListView::IconMode) {
+            setIconSize(QSize(200, 200));
+            setGridSize(QSize(-1, -1));
+            // 去除底部间距
             setViewportMargins(30, -13, -35, 0);
+            setSpacing(20);
+            if (CommonService::getInstance()->isHScreen()) {
+                setSpacing(20);
+                setViewportMargins(30, -13, -35, 0);
+            } else {
+                setSpacing(33);
+                setViewportMargins(23, -13, -35, 0);
+            }
         } else {
-            setSpacing(33);
-            setViewportMargins(23, -13, -35, 0);
+            setIconSize(QSize(36, 36));
+            setGridSize(QSize(-1, -1));
+            // 修改间距
+            setViewportMargins(10, 0, 10, 0);
+            setSpacing(0);
         }
     } else {
-        setIconSize(QSize(36, 36));
-        setGridSize(QSize(-1, -1));
-        // 修改间距
-        setViewportMargins(10, 0, 10, 0);
-        setSpacing(0);
+        if (mode == QListView::IconMode) {
+            setIconSize(QSize(150, 150));
+            setGridSize(QSize(-1, -1));
+            // 去除底部间距
+            setViewportMargins(-10, -13, -35, 0);
+            setSpacing(20);
+        } else {
+            setIconSize(QSize(36, 36));
+            setGridSize(QSize(-1, -1));
+            // 修改顶部间距
+            setViewportMargins(0, 0, 8, 0);
+            setSpacing(0);
+        }
     }
-#else
-    if (mode == QListView::IconMode) {
-        setIconSize(QSize(150, 150));
-        setGridSize(QSize(-1, -1));
-        // 去除底部间距
-        setViewportMargins(-10, -13, -35, 0);
-        setSpacing(20);
-    } else {
-        setIconSize(QSize(36, 36));
-        setGridSize(QSize(-1, -1));
-        // 修改顶部间距
-        setViewportMargins(0, 0, 8, 0);
-        setSpacing(0);
-    }
-#endif
     setViewMode(mode);
     m_viewModel = mode;
 }
@@ -569,7 +569,7 @@ void SingerListView::slotUpdatePlayingIcon()
         this->update();
     }
 }
-#ifdef TABLET_PC
+
 void SingerListView::slotHScreen(bool isHScreen)
 {
     if (this->viewMode() == QListView::IconMode) {
@@ -582,7 +582,7 @@ void SingerListView::slotHScreen(bool isHScreen)
         }
     }
 }
-#endif
+
 // 区分单双击需要，双击逻辑位置移动
 //void SingerListView::onDoubleClicked(const QModelIndex &index)
 //{
@@ -673,12 +673,21 @@ void SingerListView::slotAddSingleSong(const QString &listHash, const MediaMeta
 void SingerListView::slotRemoveSelectedSongs(const QString &deleteHash, const QStringList &musicHashs, bool removeFromLocal)
 {
     Q_UNUSED(removeFromLocal)
-    if (deleteHash != "artist" || Player::getInstance()->getCurrentPlayListHash() != "artist") {
+    QString curDeleteHash = deleteHash;
+    if (curDeleteHash != "artist" || Player::getInstance()->getCurrentPlayListHash() != "artist") {
         return;
     }
-    if (musicHashs.size() == 0) {
+    if (musicHashs.size() == 0 || Player::getInstance()->getPlayList()->isEmpty()) {
         return;
     }
+    QList<MediaMeta> *curPlayList = Player::getInstance()->getPlayList();
+    // 歌单存在非歌手歌曲不处理
+    QString firstSinger = curPlayList->first().singer;
+    for (int i = 1; i < curPlayList->size(); i++) {
+        if (curPlayList->at(i).singer != firstSinger) {
+            return;
+        }
+    }
     // 标志准备删除的歌曲,找到当前正在播放的专辑index
     int playIndex = -1;
     for (int i = 0; i < singerModel->rowCount(); i++) {
@@ -716,7 +725,10 @@ void SingerListView::slotRemoveSelectedSongs(const QString &deleteHash, const QS
     }
     // 记录切换歌单之前播放状态
     Player::PlaybackStatus preStatue = Player::getInstance()->status();
-    // 找到要播放的专辑
+    // 找到要播放的歌手,如果删除的是最后一个歌手的唯一一首歌，则从第一个播起
+    if (playIndex == (singerModel->rowCount() - 1)) {
+        playIndex = 0;
+    }
     for (int i = playIndex; i < singerModel->rowCount(); i++) {
         QModelIndex idx = singerModel->index(i, 0, QModelIndex());
         SingerInfo singerTmp = idx.data(Qt::UserRole).value<SingerInfo>();
@@ -755,7 +767,7 @@ void SingerListView::slotRemoveSelectedSongs(const QString &deleteHash, const QS
         }
     }
 }
-#ifdef TABLET_PC
+
 void SingerListView::slotPlaybackStatusChanged(Player::PlaybackStatus statue)
 {
     Q_UNUSED(statue)
@@ -763,7 +775,7 @@ void SingerListView::slotPlaybackStatusChanged(Player::PlaybackStatus statue)
         this->update();
     }
 }
-#endif
+
 void SingerListView::dragEnterEvent(QDragEnterEvent *event)
 {
     auto t_formats = event->mimeData()->formats();
@@ -797,7 +809,7 @@ void SingerListView::dropEvent(QDropEvent *event)
         auto urls = event->mimeData()->urls();
         QStringList localpaths;
         for (auto &url : urls) {
-            localpaths << url.toLocalFile();
+            localpaths << (url.isLocalFile() ? url.toLocalFile() : url.path());
         }
 
         if (!localpaths.isEmpty()) {
